from ytcc.download import Download
import argparse
import sys


def main():
    parser = argparse.ArgumentParser(description='ytgrep')
    parser.add_argument(
        '-e',
        action="store_true",
        help='Interpret PATTERN as an extended regular expression')
    parser.add_argument(
        '-v',
        action="store_true",
        help='Print debug information while searching')
    parser.add_argument(
        '-links',
        action="store_true",
        help='include shortcut links to video at matched time i.e. ?t=<time>')
    parser.add_argument('pattern', type=str, help='term to search for')
    parser.add_argument('urls', nargs='+', help='video URL(s)')

    args = parser.parse_args()
    args_dict = vars(args)

    download = Download(args_dict)
    try:
        captions = download.get_captions()
        if len(captions) == 0:
            print("No matches found.")
            sys.exit(1)
        print(captions)
    except Exception as err:
        print("Unable to retrieve captions, {}".format(err))


if __name__ == "__main__":
    main()


# related project: https://github.com/antiboredom/videogrep
# caption downloading:
# https://github.com/mkly/youtube-closed-captions/tree/master/ytcc


# packaging: https://packaging.python.org/tutorials/packaging-projects/



# -*- coding: UTF-8 -*-

import unittest
from ytcc.download import Download
from unittest.mock import patch, mock_open, Mock
from test.fixtures.webvtt import FIXTURE_WEBVTT
from colorama import Fore, Style
from ytcc.download import NoCaptionsException


def red(input):
    return Fore.RED + input + Style.RESET_ALL


class TestCaptions(unittest.TestCase):

    def test_caption(self):
        tests = [
            {
                'name': '1 video, caption found',
                'urls': ['https://www.swag.com/'],
                'pattern': 'vision',
                'regex': False,
                'links': False,
                'expected': '[00:00:17.350 --> 00:00:18.752] we have this ' + red('vision') + ' of einstein'
            },
            {
                'name': '1 video, caption not found',
                'urls': ['https://www.swag.com/'],
                'pattern': 'iwontbefound',
                'regex': False,
                'links': False,
                'expected': '',
            },
            {
                'name': '1 video, caption found more than once',
                'urls': ['https://www.swag.com/'],
                'pattern': 'light',
                'regex': False,
                'links': False,
                'expected': '[00:00:33.666 --> 00:00:38.138] actor as einstein: what ' + red('light') + ' would i see if i rode on a beam of ' + red('light') + '?',
            },
            {
                'name': '1 video, regular expression',
                'urls': ['https://www.swag.com/'],
                'pattern': 'actor|light',
                'regex': True,
                'links': False,
                'expected': '[00:00:33.666 --> 00:00:38.138] ' + red('actor') + ' as einstein: what ' + red('light') + ' would i see if i rode on a beam of ' + red('light') + '?',
            },
            {
                'name': '1 video, 1 link',
                'urls': ['https://www.swag.com/'],
                'pattern': 'actor|light',
                'regex': True,
                'links': True,
                'expected': '[00:00:33.666 --> 00:00:38.138] ' + red('actor') + ' as einstein: what ' + red('light') + ' would i see if i rode on a beam of ' + red('light') + '? (https://www.swag.com/&t=33s)',
            },
        ]
        for test in tests:
            download = Download({'urls': test['urls'],
                                 'pattern': test['pattern'],
                                 'e': test['regex'],
                                 'v': False,
                                 'links': test['links']})
            m = mock_open(read_data=FIXTURE_WEBVTT)

            with patch('ytcc.download.open', m, create=True):
                with patch('ytcc.storage.Storage.remove_file', Mock()):
                    download.get_result = Mock(return_value=0)
                    actual = download.get_captions()
                    expected = test['expected']
                    self.assertEqual(actual, expected)

    def test_caption_captions_do_not_exist(self):
        test = {
            'name': 'captions do not exist',
            'urls': ['https://www.swag.com/'],
            'pattern': 'my pattern',
            'regex': False,
            'links': False,
        }

        download = Download({'urls': test['urls'],
                             'pattern': test['pattern'],
                             'e': test['regex'],
                             'v': False,
                             'links': test['links']})
        m = mock_open(read_data=FIXTURE_WEBVTT)
        m.side_effect = FileNotFoundError

        with patch('ytcc.download.open', m, create=True):
            with patch('ytcc.storage.Storage.remove_file', Mock()):
                download.get_result = Mock(return_value=0)
                with self.assertRaises(NoCaptionsException):
                    download.get_captions()



# -*- coding: UTF-8 -*-

import unittest
from ytcc.storage import Storage
from unittest.mock import patch
import hashlib


class TestRemoveFile(unittest.TestCase):

    @patch('os.remove')
    def test_remove_file(self, mock):
        file_path = 'subtitle_' + \
            hashlib.md5('v2309jfGew'.encode('utf-8')).hexdigest() + '.en.vtt'
        video_id = 'v2309jfGew'
        storage = Storage(video_id)
        storage.remove_file()
        mock.assert_called_with(file_path)



# -*- coding: UTF-8 -*-

import unittest
from ytcc.download import Download, DownloadException, NoCaptionsException
from colorama import Fore, Style


def red(input):
    return Fore.RED + input + Style.RESET_ALL


class TestRealVideo(unittest.TestCase):

    def test_video(self):
        self.download = Download(
            {
                'urls': ['https://www.youtube.com/watch?v=jNQXAC9IVRw'],
                'pattern': 'elephants',
                'e': False,
                'v': False})
        expected = '[00:00:01.300 --> 00:00:04.400] all right, so here we are in front of the ' + \
            red('elephants') + ','
        self.assertEqual(expected, self.download.get_captions())

    def test_video_does_not_exist(self):
        self.download = Download(
            {'urls': ['12323123123'], 'pattern': 'elephants', 'e': False, 'v': False})
        with self.assertRaises(DownloadException):
            self.download.get_captions()

    def test_video_no_captions(self):
        self.download = Download(
            {
                'urls': ['https://www.youtube.com/watch?v=jyoTZ69mWZE'],
                'pattern': 'elephants',
                'e': False,
                'v': False})
        with self.assertRaises(NoCaptionsException):
            self.download.get_captions()


class TestMultipleRealVideos(unittest.TestCase):

    def test_video(self):
        self.download = Download(
            {
                'urls': [
                    'https://www.youtube.com/watch?v=jNQXAC9IVRw',
                    'https://www.youtube.com/watch?v=jNQXAC9IVRw'],
                'pattern': 'elephants',
                'e': False,
                'v': False})
        expected = """https://www.youtube.com/watch?v=jNQXAC9IVRw
[00:00:01.300 --> 00:00:04.400] all right, so here we are in front of the elephants,
https://www.youtube.com/watch?v=jNQXAC9IVRw
[00:00:01.300 --> 00:00:04.400] all right, so here we are in front of the elephants,"""
        expected = expected.replace('elephants', red('elephants'))
        self.assertEqual(expected, self.download.get_captions())

    def test_video_one_missing_captions(self):
        self.download = Download(
            {
                'urls': [
                    'https://www.youtube.com/watch?v=jNQXAC9IVRw',
                    'https://www.youtube.com/watch?v=jyoTZ69mWZE'],
                'pattern': 'elephants',
                'e': False,
                'v': False})
        expected = """https://www.youtube.com/watch?v=jNQXAC9IVRw
[00:00:01.300 --> 00:00:04.400] all right, so here we are in front of the elephants,"""
        expected = expected.replace('elephants', red('elephants'))
        self.assertEqual(expected, self.download.get_captions())



# -*- coding: UTF-8 -*-

import unittest
from ytcc.storage import Storage
import hashlib


class TestGetFilePath(unittest.TestCase):

    def test_valid(self):
        video_id = 'jNQXAC9IVRw'
        hashed_video_id = hashlib.md5(video_id.encode('utf-8')).hexdigest()
        storage = Storage(video_id)
        expected = 'subtitle_{0}.en.vtt'.format(hashed_video_id)
        self.assertEqual(expected, storage.get_file_path())



from __future__ import unicode_literals
import youtube_dl
from pycaption import WebVTTReader
from os import remove
import re
import hashlib
from ytcc.storage import Storage
from colorama import Fore, Style


class Download():
    urls = []
    search_query = ''
    regex = False
    include_links = False

    def __init__(self, args: dict, opts: dict = {}) -> None:
        self.opts = {
            'skip_download': True,
            'writeautomaticsub': True,
            'no_warnings': not args['v'],
            'quiet': not args['v'],
        }
        self.urls = args['urls']
        if args['e']:
            self.regex = True
            self.search_query = re.compile(args['pattern'])
        else:
            self.search_query = args['pattern']
        self.opts.update(opts)

        if args.get('links'):
            self.include_links = True

    def get_captions(self) -> str:

        output = ''
        for url in self.urls:
            result = self.get_result(url)
            if result != 0:
                raise Exception(
                    'Unable to download and extract captions: {0}'.format(result))
            storage = Storage(url)
            file_path = storage.get_file_path()
            try:
                with open(file_path) as f:
                    output += self.get_captions_from_output(f.read(), url)
                    storage.remove_file()
            except FileNotFoundError:
                if len(self.urls) == 1:
                    raise NoCaptionsException("no captions found.")
                else:
                    # make this orange like the youtube-dl warnings
                    print("WARNING: no captions found for {}".format(url))

        # remove final newline
        if len(output) > 0 and output[-1] == '\n':
            output = output[:-1]
        return output

    def get_result(self, video_id: str) -> int:
        self.opts['outtmpl'] = 'subtitle_' + \
            hashlib.md5(str(video_id).encode('utf-8')).hexdigest()
        with youtube_dl.YoutubeDL(self.opts) as ydl:
            try:
                return ydl.download([video_id])  # J
            except youtube_dl.utils.DownloadError as err:
                raise DownloadException(
                    "Unable to download captions: {0}".format(str(err)))
            except youtube_dl.utils.ExtractorError as err:
                raise DownloadException(
                    "Unable to extract captions: {0}".format(str(err)))
            except Exception as err:
                raise DownloadException(
                    "Unknown exception downloading and extracting captions: {0}".format(
                        str(err)))

    def get_captions_from_output(self, output: str, url: str) -> str:
        reader = WebVTTReader()

        captions = []
        for caption in reader.read(output).get_captions('en-US'):
            stripped = self.remove_time_from_caption(
                url, str(caption).replace(r'\n', " "))
            stripped += "\n"
            captions.append(stripped)

        return self.process_captions(captions, url)

    def get_time_url(self, url, time_str):
        h, m, s = time_str.split(':')
        seconds = str(int(h) * 3600 + int(m) * 60 + int(s))
        return url + '&t=' + str(seconds) + 's'

    def process_captions(self, captions, url):
        temp_final = ''
        # if we have multiple urls, print the URL at the beginning
        if len(self.urls) > 1:
            temp_final = url + '\n'
        i = -1
        for caption in captions:
            i += 1
            stripped = caption.lower()
            # temporarily remove time prefix via slicing (the time prefix is
            # stable)
            prefix = stripped[0:32]
            stripped = stripped[32:]
            # remove duplicate entries

            if self.regex:
                l = self.search_query.findall(stripped)
                if len(l) > 0:
                    for match in l:
                        if Fore.RED + match + Style.RESET_ALL not in stripped:
                            stripped = stripped.replace(
                                match, Fore.RED + match + Style.RESET_ALL)
                            stripped = stripped.replace("'", "").strip()
                    stripped = prefix + stripped
                    if self.include_links:
                        start_time = prefix[1:9]
                        time_url = self.get_time_url(url, start_time)
                        stripped = stripped.rstrip() + ' (' + time_url + ')'
                    temp_final += stripped

            elif self.search_query in stripped:

                # It's possible that we have duplicate entries, such as:
                # [00:45:15.960 --> 00:45:15.970] will do this topological sort is what'
                # [00:45:15.970 --> 00:45:20.430] will do this topological sort is what the selvam is usually called topological'
                # so skip the original duplicate if we find a match like this. We trim and ignore quotes to avoid
                # whitespace and quotes from stopping what would otherwise be a
                # match

                if i < len(captions) - 1 and stripped.strip().replace("'",
                                                                      "").replace('"',
                                                                                  '') in str(captions[i + 1]).strip().replace("'",
                                                                                                                              "").replace('"',
                                                                                                                                          ''):
                    continue
                stripped = stripped.replace("'", "").strip()
                stripped = stripped.replace(
                    self.search_query,
                    Fore.RED +
                    self.search_query +
                    Style.RESET_ALL)
                stripped = prefix + stripped
                if self.include_links:
                    start_time = prefix[1:9]
                    time_url = self.get_time_url(url, start_time)
                    stripped = stripped.rstrip() + ' (' + time_url + ')'
                temp_final += stripped + '\n'

        return temp_final

    def remove_time_from_caption(self, video_id: str, caption: str) -> str:
        caption = re.sub(
            r"(\d{2}:\d{2}:\d{2}.\d{3} --> \d{2}:\d{2}:\d{2}.\d{3})",
            r"[\1]",
            caption,
            flags=re.DOTALL)
        # remove first char from string (will be a quote)
        return caption[1:]


class DownloadException(Exception):

    def __init__(self, *args, **kwargs):
        Exception.__init__(self, *args, **kwargs)


class NoCaptionsException(Exception):

    def __init__(self, *args, **kwargs):
        Exception.__init__(self, *args, **kwargs)



# -*- coding: UTF-8 -*-

import unittest
from ytcc.download import Download
from unittest.mock import patch, Mock


class TestGetResult(unittest.TestCase):

    def test_result_is_zero(self):
        video_id = 'we342oij234'
        ydl = Mock()
        ydl.download = Mock(return_value=0)

        with patch('youtube_dl.YoutubeDL.__enter__', return_value=ydl):
            download = Download(
                {
                    'urls': ['http://www.youtube.com/watch?v=we342oij234'],
                    'pattern': 'asdf',
                    'e': False,
                    'v': False})
            self.assertEqual(0, download.get_result(video_id))
            ydl.download.assert_called_with(
                ['we342oij234'])



# -*- coding: UTF-8 -*-

import re
import os
import hashlib


class Storage():

    def __init__(self, video_url: str) -> None:
        self.video_url = video_url

    def get_file_path(self) -> str:
        return 'subtitle_{0}.en.vtt'.format(re.sub(
            r'[^\w-]', '', hashlib.md5(str(self.video_url).encode('utf-8')).hexdigest()))

    def remove_file(self) -> None:
        os.remove(self.get_file_path())



#!/usr/bin/env python
# -*- coding: UTF-8 -*-

from setuptools import setup, find_packages

with open('LICENSE') as f:
    license = f.read()

with open('README.rst') as f:
    readme = f.read()

setup(
    name='ytgrep',
    version='0.4.0',
    description='CLI tool to search youtube captions',
    long_description=readme,
    author='Alex Kohler',
    author_email='alexjohnkohler@gmail.com',
    license=license,
    packages=find_packages(exclude=('test')),
    py_modules=['ytgrep'],
    entry_points={
        "console_scripts": ['ytgrep = ytgrep:main']
    },
    classifiers=[
        'Programming Language :: Python :: 3.7',
    ]
)
